package ch.unige.solidify.exception;

import static org.springframework.http.HttpStatus.BAD_REQUEST;
import static org.springframework.http.HttpStatus.PRECONDITION_FAILED;

import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.web.client.DefaultResponseErrorHandler;

import com.fasterxml.jackson.databind.ObjectMapper;

import ch.unige.solidify.repository.SolidifyDbLockError;
import ch.unige.solidify.repository.SolidifyError;

public class RestResponseErrorHandler extends DefaultResponseErrorHandler {
  private static final Logger log = LoggerFactory.getLogger(RestResponseErrorHandler.class);

  @Override
  public void handleError(ClientHttpResponse response) throws IOException {
    final HttpStatus statusCode = HttpStatus.resolve(response.getRawStatusCode());
    if (statusCode == PRECONDITION_FAILED || statusCode == BAD_REQUEST) {
      final String body = this.getResponseBodyString(response);
      // if 412, used for Lock Exception
      if (statusCode == PRECONDITION_FAILED) {
        // first try to convert to LockException
        final SolidifyDbLockError lockErrors = this.convertResponseBody2Exception(body, SolidifyDbLockError.class);
        if (lockErrors != null) {
          throw new SolidifyRestException("Lock Exception", lockErrors);
        }
      }
      // second, try to convert to SolidifyRestError
      final SolidifyError error = this.convertResponseBody2Exception(body, SolidifyError.class);
      if (error != null) {
        throw new SolidifyRestException("Validation Exception", error);
      }
    }
    super.handleError(response);
  }

  protected <T> T convertResponseBody2Exception(String body, Class<T> clazz) {
    try {
      final ObjectMapper objectMapper = new ObjectMapper();
      return objectMapper.readValue(body, clazz);
    } catch (final Exception e) {
      log.error("Deserialization error: target class =" + clazz + "JsonString=" + body + "", e);
      return null;
    }

  }

  protected String getResponseBodyString(ClientHttpResponse response) {
    final byte[] bodyRawDate = this.getResponseBody(response);
    final Charset charset = this.getCharset(response);
    if (charset != null) {
      return new String(bodyRawDate, charset);
    }
    return new String(bodyRawDate, StandardCharsets.UTF_8);
  }
}
