package ch.unige.solidify.service;

import java.security.Principal;
import java.text.ParseException;
import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Component;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.context.request.RequestContextHolder;

import com.nimbusds.jwt.SignedJWT;

@Component
public class SolidifyHttpRequestInfoProvider implements HttpRequestInfoProvider {
  private static final Logger log = LoggerFactory.getLogger(SolidifyHttpRequestInfoProvider.class);

  private HttpServletRequest incomingRequest;

  private SolidifyHttpRequestInfoProvider(HttpServletRequest incomingRequest){
    this.incomingRequest = incomingRequest;
  }

  public MultiValueMap<String, String> getAuthorizationHeader() {
    final MultiValueMap<String, String> authorizationHeader = new LinkedMultiValueMap<>();
    if (RequestContextHolder.getRequestAttributes() != null) {
      log.debug("Request with attributes");
      if (incomingRequest.getHeader(HttpHeaders.AUTHORIZATION) != null) {
        if(log.isDebugEnabled()) {
          try {
            String token = incomingRequest.getHeader(HttpHeaders.AUTHORIZATION);
            log.debug("Authorization header present {}", SignedJWT.parse(token.substring("Bearer ".length())).getJWTClaimsSet());
          } catch (ParseException e) {
            log.debug("Invalid token, cannot parse token");
          }
        }
        authorizationHeader.add(HttpHeaders.AUTHORIZATION, incomingRequest.getHeader(HttpHeaders.AUTHORIZATION));
      } else {
        log.debug("Authorization header not present");
      }
    } else {
      log.debug("Request attributes is null");
    }
    return authorizationHeader;
  }

  public Principal getPrincipal() {
    if (RequestContextHolder.getRequestAttributes() != null) {
      return incomingRequest.getUserPrincipal();
    }
    return null;
  }
}
