package ch.unige.solidify.test;

import static ch.unige.solidify.test.helper.AbstractResourceDoc.TEST_STRING_VALUE;
import static org.springframework.restdocs.payload.PayloadDocumentation.fieldWithPath;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.json.JSONObject;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.springframework.restdocs.payload.FieldDescriptor;
import org.springframework.restdocs.snippet.Attributes.Attribute;

import ch.unige.solidify.test.helper.TestObjectBuilder;

public class TestObjectBuilderTest {

  private List<FieldDescriptor> fieldDescriptorList;

  @Before
  public void setUp() {
    // @formatter:off
    this.fieldDescriptorList = Arrays.asList(
            fieldWithPath("a.b").attributes(new Attribute(TEST_STRING_VALUE, "1")),

            fieldWithPath("a.d.b").attributes(new Attribute(TEST_STRING_VALUE, "2")),

            fieldWithPath("c.d.b").attributes(new Attribute(TEST_STRING_VALUE, "3")),

            fieldWithPath("c.d.c").attributes(new Attribute(TEST_STRING_VALUE, "4")));

    // @formatter:on
  }

  @Test
  public void testEmptyObject() {
    final JSONObject jsonObject = TestObjectBuilder.buildJsonObject(Collections.emptyList());
    Assert.assertEquals("{}", jsonObject.toString());
  }

  @Test
  public void testJsonBuild() {
    final JSONObject jsonObject = TestObjectBuilder.buildJsonObject(this.fieldDescriptorList);
    Assert.assertEquals("1", jsonObject.getJSONObject("a").get("b"));
    Assert.assertEquals("2", jsonObject.getJSONObject("a").getJSONObject("d").get("b"));
    Assert.assertEquals("3", jsonObject.getJSONObject("c").getJSONObject("d").get("b"));
    Assert.assertEquals("4", jsonObject.getJSONObject("c").getJSONObject("d").get("c"));
  }
}
