package ch.unige.solidify.listener;

import javax.persistence.PostPersist;
import javax.persistence.PostRemove;
import javax.persistence.PostUpdate;

import org.springframework.context.ApplicationEventPublisher;

import ch.unige.solidify.config.SolidifyEventPublisher;
import ch.unige.solidify.message.CacheMessage;
import ch.unige.solidify.rest.CompositeResource;
import ch.unige.solidify.rest.Resource;

public class CacheListener {

  @PostPersist
  private void notifyPersist(Object object) {

    final ApplicationEventPublisher publisher = SolidifyEventPublisher.getPublisher();

    /*
     * ApplicationEventPublisher may be null when creating resources during initialization
     */
    if (publisher != null) {
      CacheMessage cacheMessage = null;
      if (object instanceof CompositeResource) {
        cacheMessage = ((CompositeResource<?, ?>) object).getCacheMessage();
      }
      if (cacheMessage != null) {
        publisher.publishEvent(cacheMessage);
      }
    }
  }

  @PostUpdate
  @PostRemove
  private void notifyUpdate(Object object) {

    final ApplicationEventPublisher publisher = SolidifyEventPublisher.getPublisher();

    /*
     * ApplicationEventPublisher may be null when creating resources during initialization
     */
    if (publisher != null) {
      CacheMessage cacheMessage = null;
      if (object instanceof Resource) {
        cacheMessage = ((Resource<?>) object).getCacheMessage();
      } else if (object instanceof CompositeResource) {
        cacheMessage = ((CompositeResource<?, ?>) object).getCacheMessage();
      }
      if (cacheMessage != null) {
        publisher.publishEvent(cacheMessage);
      }
    }

  }

}
