package ch.unige.solidify.util;

import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.context.event.ApplicationPreparedEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.core.env.ConfigurableEnvironment;
import org.springframework.core.env.EnumerablePropertySource;
import org.springframework.core.env.PropertySource;

public class PropertiesLogger implements ApplicationListener<ApplicationPreparedEvent> {
  private static final Logger log = LoggerFactory.getLogger(PropertiesLogger.class);

  private ConfigurableEnvironment environment;
  private boolean isPrinted;

  @Override
  public void onApplicationEvent(ApplicationPreparedEvent event) {
    if (!this.isPrinted) {
      this.environment = event.getApplicationContext().getEnvironment();
      this.printProperties();
      this.isPrinted = true;
    }
  }

  public void printProperties() {
    for (final EnumerablePropertySource<?> propertySource : this.findPropertiesPropertySources()) {
      log.info("********** {} **********", propertySource.getName());
      final String[] propertyNames = propertySource.getPropertyNames();
      Arrays.sort(propertyNames);
      for (final String propertyName : propertyNames) {
        final String resolvedProperty = this.environment.getProperty(propertyName);
        final String sourceProperty = propertySource.getProperty(propertyName).toString();
        if (propertyName.toLowerCase().endsWith("password") ||
                propertyName.toLowerCase().endsWith("secret") ||
                propertyName.toLowerCase().endsWith("secret-key") ||
                propertyName.toLowerCase().endsWith("access-key") ||
                propertyName.toLowerCase().endsWith("apikey") ||
                propertyName.toLowerCase().endsWith("users.root") ||
                propertyName.toLowerCase().endsWith("cloud.config.uri")) {
          log.info("{}={}", propertyName, "********");
          continue;
        }
        if (resolvedProperty.equals(sourceProperty)) {
          log.info("{}={}", propertyName, resolvedProperty);
        } else {
          log.info("{}={} OVERRIDEN to {}", propertyName, sourceProperty, resolvedProperty);
        }
      }
    }
  }

  private List<EnumerablePropertySource<?>> findPropertiesPropertySources() {
    final List<EnumerablePropertySource<?>> propertiesPropertySources = new LinkedList<>();
    for (final PropertySource<?> propertySource : this.environment.getPropertySources()) {
      if (propertySource instanceof EnumerablePropertySource) {
        propertiesPropertySources.add((EnumerablePropertySource<?>) propertySource);
      }
    }
    return propertiesPropertySources;
  }
}
